%% BetaChainDist
% This routine calculates the pitch domain distance between all dyads
% (intervals), or triads, or tetrads of a "reference" pitch class
% collection and of a scale generated by two intervals - a "period" and a
% "generator". The size of the generator can be iterated over a range of
% tunings. 

% In this routine, the dyads, triads, and tetrads are embedded as relative
% pitches (which are invariant to transposition), so it can be used to find
% tunings of regular temperaments (e.g., by using a JI major triad as the
% reference triad, and observing generator tunings that are close to it).

% These distances are calculated from embeddings in the pitch domain (the
% range is the expected number of tones heard). This is unlike the familiar
% voice-leading distances that are calculated from embeddings in the voice
% domain (e.g., bass, tenor, alto, soprano), and whose ranges are pitch.
% Pitch domain embeddings allow distances to be calculated between pitch
% collections with differing numbers of tones, and are more appropriate
% when tones cannot be simply categorised. See the paper below for a fuller
% discussion.

% Authors: Andrew Milne, William Sethares
% Revision: 1.10  Date: 2011/06/24

% Reference: Milne, A.J., Sethares, W.A., Laney, R., Sharp, D.B. (2011)
% Modelling the Similarity of Pitch Collections with Expectation Tensors,
% Journal of Mathematics and Music, 5(1), 1-20.

%% Preliminaries
clear
x_w = 0;
y_w = 0;

%% Some useful pitch collections
Equal12 = [0, 100, 200, 300, 400, 500, 600, 700, 800, 900, 1000, 1100];
Major12tet = [0, 200, 400, 500, 700, 900, 1100];
Aeolian12tet = [0, 200, 300, 500, 700, 800, 1000];
HarmonicMin12tet = [0, 200, 300, 500, 700, 800, 1100];
Pentatonic12tet = [0, 200, 400, 700, 900];
MajTriad12tet = [0, 400, 700];
MinTriad12tet = [0, 300, 700];
JIMajorScale = [0, 204, 386, 498, 702, 884, 1088];
JIMajorPentad = [0, 386.3, 702, 968.8, 551.3];
JIMajorTetrad = [0, 386.3, 702, 968.8];
JIMajorTriad = [0, 386.3, 702];
magicJI = [854.75, 41.06, 427.37, 813.69, 0, 356.70, 743.01, 1129.33, 315.64, 701.96];
eb19 = [0, 76.97 132.25 209.21 286.18 341.46 418.43 495.39 550.68 572.36 627.64 704.61 781.57 836.86 913.82 990.79 1046.07 1067.75 1123.03];
Bagpipe = [0, 197, 341, 495, 703, 853, 1009];
BPTriad = [0, 884.36, 1466.87];

%% Parameters
% "period" is the size in cents of the octave (or other period). The
% standard octave is 1200 cents. All entered pitch values are reduced to 
% lie in the interval [0, period).
% "q" = the resolution (granularity) of the pitch embedding (in divisions 
% of the period). A higher value (e.g., 1200) allows for more accurate
% modelling of perceptual uncertainty, and the distances between closely 
% spaced pitches, but increases calculation and memory requirements. If 
% modelling standard 12-EDO pitch classes, a value of 12 is adequate.
% "sigma" = standard deviation, in cents, of the Gaussian probability
% function
% "win_len" = the length of the smoothing kernel (in standard deviations)
% "r" = 1, 2, 3, or 4 - i.e., monad, dyad, triad, or tetrad embedding. Note that
% the procedure runs at O(d(q^r)), where d is the number of elements in x_pc 
% (or y_pc). For higher values of r or d, it may be necessary to reduce q, or 
% the size of gen_range or the size of the increment. 
% "metric" = 'l_p' or 'cosine' 
% "p" = size of the norm when using l_p metrics.
period = 1200;
q = 1200;
sigma = 6;
win_len = 6;
r = 2;
metric = 'cosine';
p = 1;

%% Pitch class collections x and the generated scales.
% "x_pc" = the pitches (cents) of the tone collection we wish to compare
% with the generated scales. 
% "x_w" = the respective saliences of the "x_pc" tones. If nothing or 0 is 
% entered for x_w, all saliences are set to 1. 
% "gen_range" = the range (and increment) of generator sizes (cents). Note
% that the sie of the period is set in the above Parameters section. 
% "cardinality" = the number of tones in the generated scale.
x_pc = [0, 386.3, 702, 968.8];
x_w  = [1/2, 1/5, 1/3, 1/7];
gen_range = 0:1:1200;
cardinality = 12;

%% Calculations of chain and metrics between different expectation arrays
chainlength = 0:cardinality-1;
d = zeros(1,length(gen_range));
i = 1;

for gen = gen_range;
    chain = gen.*chainlength;
    d(i) = ExpectationMetrics(period,q,metric,p,sigma,win_len,x_pc,x_w,chain,0,'rel',r);
i=i+1;
end

%% Plotting
figure(4), subplot(3,1,3), plot(gen_range,d)
set(gca,'XTick',0:100:max(gen_range))
[distance, tuning] = min(d)

figure(10), polar(2*pi*gen_range/period,d)
view(90,270)
ph=findall(gca,'type','text');
ps=get(ph,'string');
ps(1:15)={''};
ps([4,5,6,7,8,9,10,11,12,13,14,15])={
    int2str(0)
    int2str(6*period/12)
    int2str(11*period/12)
    int2str(5*period/12)
    int2str(10*period/12)
    int2str(4*period/12)
    int2str(9*period/12)
    int2str(3*period/12)
    int2str(8*period/12)
    int2str(2*period/12)
    int2str(7*period/12)
    int2str(1*period/12)
    };
set(ph,{'string'},ps);
