%% TonesSpectralDist
% This routine calculates "spectral pitch distance" - the pitch domain
% distance between the partials of two tones or chords. 

% In this routine, spectral pitch distances are calculated between a
% "reference" tone and a variety of differently tuned "continuum" tones. In
% the resulting figure: the x-axis is the pitch of the continuum tone
% relative to the reference tone; the y-axis is the spectral pitch distance
% between the reference and continuum tones.

% These distances are calculated from embeddings in the pitch domain (the
% range is the expected number of tones heard). This is unlike the familiar
% voice-leading distances that are calculated from embeddings in the voice
% domain (e.g., bass, tenor, alto, soprano), and whose ranges are pitch.
% Pitch domain embeddings allow distances to be calculated between pitch
% collections with differing numbers of tones, and are more appropriate
% when tones (in this case partials) cannot be simply categorised. See the
% paper below for a fuller discussion.

% Author: Andrew Milne 
% Revision: 1.10  Date: 2011/06/24

% Reference: Milne, A.J., Sethares, W.A., Laney, R., Sharp, D.B. (2011)
% Modelling the Similarity of Pitch Collections with Expectation Tensors,
% Journal of Mathematics and Music, 5(1), 1-20.

%% Preliminaries
clear

%% Parameters
% "period" is the size in cents of the octave (or other period). The
% standard octave is 1200 cents. All entered pitch values are reduced to 
% lie in the interval [0, period).
% "q" = the resolution (granularity) of the pitch embedding (in divisions 
% of the period). A higher value (e.g., 1200) allows for more accurate
% modelling of perceptual uncertainty, and the distances between closely 
% spaced pitches, but increases calculation and memory requirements. If 
% modelling standard 12-EDO pitch classes, a value of 12 is adequate.
% "metric" = 'l_p' or 'cosine' 
% "p" = size of the norm when using l_p metrics.
% "sigma" = standard deviation, in cents, of the Gaussian probability
% function
% "win_len" = the length of the smoothing kernel (in standard deviations)
period = 1200;
q = 1200;
metric = 'cosine';
p = 1;
sigma = 3;
win_len = 6;

%% Reference and generated triad parameters
% "low_limit" and "high_limit" sets the range (in cents) of the continuum
% triad relative to the reference triad 
% "inc" sets the size, in cents, of the increments between low_limit and 
% high_limit 
% "num_harmonics" sets the number of spectral pitches in each tone 
% "rolloff" sets the amplitude of each tone's harmonics to 1/n^rolloff, 
% where n is the harmonic number; rolloff = 0 gives a flat spectrum (all 
% saliences are 1; rolloff = 1 gives a sawtooth spectrum; rolloff = a very 
% big number results in all harmonics, except the fundamental, having zero 
% salience (though this is easier to achieve by setting num_harmonics to 1).
% "RefTriad" - can be either 'Major' or 'Minor' 
% "Gamma" - the amount of tempering of the reference triad's tones away 
% from just intonation (0 is tempered, 1 is just)
% "TuningContinuum" = 'Syntonic', 'Magic', or 'Hanson' - various ways of 
% generating the reference triad from two generators alpha and beta. To set
% the size of alpha and beta look inside the if loops in the Generate
% reference triad section.
low_limit = 0;
high_limit = 1200;
inc = 1;
num_harmonics = 10;
rolloff = 0;

%% Generate reference triad's spectrum
SpectrumX = zeros(1, num_harmonics);
for j = 1:num_harmonics
    SpectrumX(j) = 1200*log2(j);
end

SpectrumX_w = zeros(1, num_harmonics);
for j = 1:num_harmonics
    SpectrumX_w(j) = 1/(j^rolloff);
end

%% Generate continuum triads and send to embeddings
SpectrumY = SpectrumX;
SpectrumY_w = SpectrumX_w;

shifts = low_limit:inc:high_limit;
Dist = zeros(1,length(shifts));
for i = 1:length(shifts)
    SpectrumY(1:num_harmonics) = SpectrumX(1:num_harmonics)+shifts(i);
    Dist(i) = ExpectationMetrics(period,q,metric,p,sigma,win_len,SpectrumX,SpectrumX_w,SpectrumY,SpectrumY_w,'abs',1);
end

%% Plotting
figure(1), subplot(3,1,3)
stairs(Dist)
grid on
axis([1,length(Dist),0,1.2])
set(gca,'XTick',1:(length(shifts)-1)*100*q/(period*(high_limit - low_limit)) :length(shifts))
set(gca,'XTickLabel',low_limit:(q/12):high_limit)

theta = 2*pi*(low_limit:inc:high_limit)/period;
figure(10), polar(theta, Dist)
view(90,270)
ph=findall(gca,'type','text');
ps=get(ph,'string');
ps(1:15)={''};
ps([4,5,6,7,8,9,10,11,12,13,14,15])={
    int2str(0)
    int2str(6*period/12)
    int2str(11*period/12)
    int2str(5*period/12)
    int2str(10*period/12)
    int2str(4*period/12)
    int2str(9*period/12)
    int2str(3*period/12)
    int2str(8*period/12)
    int2str(2*period/12)
    int2str(7*period/12)
    int2str(1*period/12)
    };
set(ph,{'string'},ps);
