%% TriadSpectralDist
% This routine calculates "spectral pitch distance" - the pitch domain
% distance between the partials of two tones or chords. 

% In this routine, spectral pitch distances are calculated between a
% "reference" triad and a variety of differently tuned "continuum" triads
% (all of which contain the same sized fifth as the reference triad). In
% the resulting figure: the x-axis is the pitch of the "root" and "fifth"
% of the continuum triad relative to the reference triad's root and fifth;
% the y-axis is the pitch of the "third" of the continuum triad relative to
% the reference triad's third; the z-axis (colour) is the spectral pitch
% distance between the reference and continuum triads.

% These distances are calculated from embeddings in the pitch domain (the
% range is the expected number of tones heard). This is unlike the familiar
% voice-leading distances that are calculated from embeddings in the voice
% domain (e.g., bass, tenor, alto, soprano), and whose ranges are pitch.
% Pitch domain embeddings allow distances to be calculated between pitch
% collections with differing numbers of tones, and are more appropriate
% when tones (in this case partials) cannot be simply categorised. See the
% paper below for a fuller discussion.

% Author: Andrew Milne
% Revision: 1.10  Date: 2011/06/24

% Reference: Milne, A.J., Sethares, W.A., Laney, R., Sharp, D.B. (2011)
% Modelling the Similarity of Pitch Collections with Expectation Tensors,
% Journal of Mathematics and Music, 5(1), 1-20.

%% Preliminaries
clear

%% Parameters
% "period" is the size in cents of the octave (or other period). The
% standard octave is 1200 cents. All entered pitch values are reduced to 
% lie in the interval [0, period).
% "q" = the resolution (granularity) of the pitch embedding (in divisions 
% of the period). A higher value (e.g., 1200) allows for more accurate
% modelling of perceptual uncertainty, and the distances between closely 
% spaced pitches, but increases calculation and memory requirements. If 
% modelling standard 12-EDO pitch classes, a value of 12 is adequate.
% "metric" = 'l_p' or 'cosine' 
% "p" = size of the norm when using l_p metrics.
% "sigma" = standard deviation, in cents, of the Gaussian probability
% function
% "win_len" = the length of the smoothing kernel (in standard deviations)
period = 1200;
q = 1200;
metric = 'cosine';
p = 1;
sigma = 3;
win_len = 6;

%% Reference and generated triad parameters
% "low_limit" and "high_limit" sets the range (in cents) of the continuum
% triad relative to the reference triad 
% "inc" sets the size, in cents, of the increments between low_limit and 
% high_limit 
% "num_harmonics" sets the number of spectral pitches in each tone 
% "rolloff" sets the amplitude of each tone's harmonics to 1/n^rolloff, 
% where n is the harmonic number; rolloff = 0 gives a flat spectrum (all 
% saliences are 1; rolloff = 1 gives a sawtooth spectrum; rolloff = a very 
% big number results in all harmonics, except the fundamental, having zero 
% salience (though this is easier to achieve by setting num_harmonics to 1).
% "RefTriad" - can be either 'Major' or 'Minor' 
% "Gamma" - the amount of tempering of the reference triad's tones away 
% from just intonation (0 is tempered, 1 is just)
% "TuningContinuum" = 'Syntonic', 'Magic', or 'Hanson' - various ways of 
% generating the reference triad from two generators alpha and beta. To set
% the size of alpha and beta look inside the if loops in the "Generate
% reference triad" section.
low_limit = -600;
high_limit = 700;
inc = 100;
num_harmonics = 10;
rolloff = 0;
RefTriad = 'Major';
Gamma = 0;
TuningContinuum ='Syntonic';
X_tone_weights = [1, 1, 1];
Y_tone_weights = [1, 1, 1];

%% Generate reference triad
if isequal(TuningContinuum,'Syntonic');
    Alpha=1200;
%     Beta=696.5784;
    Beta=700;
    if isequal(RefTriad,'Major');
        ToneX1=0;
        ToneX3=((-2*Alpha)+(4*Beta))+(Gamma*((1200*log2(5/4))-((-2*Alpha)+(4*Beta))));
        ToneX5=((0*Alpha)+(1*Beta))+(Gamma*((1200*log2(3/2))-((0*Alpha)+(1*Beta))));
    else isequal(RefTriad,'Minor');
        ToneX1=0;
        ToneX3=((2*Alpha)+(-3*Beta))+(Gamma*((1200*log2(6/5))-((2*Alpha)+(-3*Beta))));
        ToneX5=((0*Alpha)+(1*Beta))+(Gamma*((1200*log2(3/2))-((0*Alpha)+(1*Beta))));
    end
elseif isequal(TuningContinuum,'Magic');
    Alpha=1200;
    Beta=6*1200/19;
    if isequal(RefTriad,'Major');
        ToneX1=0;
        ToneX3=((0*Alpha)+(1*Beta))+(Gamma*((1200*log2(5/4))-((0*Alpha)+(1*Beta))));
        ToneX5=((-1*Alpha)+(5*Beta))+(Gamma*((1200*log2(3/2))-((-1*Alpha)+(5*Beta))));
    else isequal(RefTriad,'Minor');
        ToneX1=0;
        ToneX3=((-1*Alpha)+(4*Beta))+(Gamma*((1200*log2(6/5))-((-1*Alpha)+(4*Beta))));
        ToneX5=((-1*Alpha)+(5*Beta))+(Gamma*((1200*log2(3/2))-((-1*Alpha)+(4*Beta))));
    end
else isequal(TuningContinuum,'Hanson');
end

%% Generate reference triad's spectrum
ChordX = [ToneX1 ToneX3 ToneX5];
SpectrumX = zeros(1, 3*num_harmonics);
for i = 1:3
    for j = 1:num_harmonics
        SpectrumX(((i-1)*num_harmonics)+j) = ChordX(i) + (1200*log2(j));
    end
end

SpectrumX_w = zeros(1, 3*num_harmonics);
for i = 1:3
    for j = 1:num_harmonics
        SpectrumX_w(((i-1)*num_harmonics)+j) = X_tone_weights(i)/(j^rolloff);
    end
end

%% Generate continuum triads and send to embeddings
SpectrumY = zeros(1,3*num_harmonics);
SpectrumY_w = zeros(1, 3*num_harmonics);
for i = 1:3
    for j = 1:num_harmonics
        SpectrumY_w(((i-1)*num_harmonics)+j) = Y_tone_weights(i)/(j^rolloff);
    end
end

shifts = low_limit:inc:high_limit;
Dist = zeros(length(shifts),length(shifts));
for i = 1:length(shifts)
    SpectrumY(1:num_harmonics) = SpectrumX(1:num_harmonics)+shifts(i);
    SpectrumY(2*num_harmonics+1:3*num_harmonics) = SpectrumX(2*num_harmonics+1:3*num_harmonics)+shifts(i);
    for j = 1:length(shifts)
        SpectrumY(num_harmonics+1:2*num_harmonics) = SpectrumX(num_harmonics+1:2*num_harmonics)+shifts(j);
        Dist(j,i) = ExpectationMetrics(period,q,metric,p,sigma,win_len,SpectrumX,SpectrumX_w,SpectrumY,SpectrumY_w, 'abs',1);
    end
end

% This reversal allows for easy comparison with Krumhansl's tonal hierarchies
% Dist = 1-Dist;

%% Plotting
clf
hold off
figure(10)
surf(1:length(shifts),1:length(shifts),Dist,'FaceAlpha',1,'LineStyle','none')
view(0,90)
axis([1 length(shifts) 1 length(shifts)])
colormap bone
lighting phong
grid off
set(gca,'PlotBoxAspectRatio',[2^(1/2),1,0.5])
set(gca,'XTick',1: (length(shifts)-1)*100*q/(period*(high_limit - low_limit)) :length(shifts))
set(gca,'YTick',1: (length(shifts)-1)*100*q/(period*(high_limit - low_limit)) :length(shifts))
set(gca,'XTickLabel',low_limit:(q/12):high_limit)
set(gca,'YTickLabel',low_limit:(q/12):high_limit)
title(RefTriad)
xlabel('Roots and fifths')
ylabel('Thirds')
zlabel('Distance')
grid off