%% n-TETsDist
% This routine calculates the pitch domain distance between all dyads
% (intervals), or triads, or tetrads, of a "reference" pitch class
% collection and of a variety of different n-EDOs (equal divisions of the
% octave). The size of n can be iterated over a range of values.

% In this routine, the dyads, triads, and tetrads are embedded as relative
% pitches, which are invariant to transposition, so it can be used to find
% equal temperaments of just intonation (e.g., by using a JI major triad as
% the reference triad, and observing n-EDOs that are close to it).

% These distances are calculated from embeddings in the pitch domain (the
% range is the expected number of tones heard). This is unlike the familiar
% voice-leading distances that are calculated from embeddings in the voice
% domain (e.g., bass, tenor, alto, soprano), and whose ranges are pitch.
% Pitch domain embeddings allow distances to be calculated between pitch
% collections with differing numbers of tones, and are more appropriate
% when tones cannot be simply categorised. See the paper below for a fuller
% discussion.

% Authors: Andrew Milne, William Sethares 
% Revision: 1.10  Date: 2011/06/24

% Reference: Milne, A.J., Sethares, W.A., Laney, R., Sharp, D.B. (2011)
% Modelling the Similarity of Pitch Collections with Expectation Tensors,
% Journal of Mathematics and Music, 5(1), 1-20.

%% Preliminaries
clear
tic
x_w = 0;
y_w = 0;

%% Some useful pitch collections
Major12tet = [0, 200, 400, 500, 700, 900, 1100];
Aeolian12tet = [0, 200, 300, 500, 700, 800, 1000];
HarmonicMin12tet = [0, 200, 300, 500, 700, 800, 1100];
MajTriad12tet = [0, 400, 700];
MinTriad12tet = [0, 300, 700];
Equal12 = [0, 100, 200, 300, 400, 500, 600, 700, 800, 900, 1000, 1100];
JIMajorTriad = [0, 386.3, 702];
JIMajorTetrad = [0, 386.3, 702, 1088.3];
JIMajorScale = [0, 204, 386, 498, 702, 884, 1088];
pentatonic12tet = [0, 200, 400, 700, 900];
eb19 = [0, 76.96545 132.24837 209.21382 286.17927 341.46219 418.42764 495.39309 550.67601 572.35854 627.64146 704.60691 781.57236 836.85528 913.82073 990.78618 1046.06910 1067.7516 1123.03455];
magicJI = [854.74514457, 41.0588584, 427.37257227, 813.68628617, 0, 356.70014537, 743.01385927, 1129.32757277, 315.64128697, 701.95500087];
JIMajorTetrad600 = [0, 386, 102, 369];
JIMajorPentad = [0, 386, 702, 968.8, 551.3];
Bagpipe = [0, 197, 341, 495, 703, 853, 1009];
JIMajorTriadhalved = [0, 193, 351];
BPTriad = [0, 884, 1467];

%% Parameters
% "period" is the size in cents of the octave (or other period). The
% standard octave is 1200 cents. All entered pitch values are reduced to 
% lie in the interval [0, period).
% "q" = the resolution (granularity) of the pitch embedding (in divisions 
% of the period). A higher value (e.g., 1200) allows for more accurate
% modelling of perceptual uncertainty, and the distances between closely 
% spaced pitches, but increases calculation and memory requirements. If 
% modelling standard 12-EDO pitch classes, a value of 12 is adequate.
% "sigma" = standard deviation, in cents, of the Gaussian probability
% function
% "win_len" = the length of the smoothing kernel (in standard deviations)
% "r" = 1, 2, 3, or 4 - i.e., monad, dyad, triad, or tetrad embedding. Note
% that the procedure runs at O(d(q^r)), where d is the number of elements
% in x_pc (or y_pc). For higher values of r or d, it may be necessary to
% reduce q.
% "metric" = 'l_p' or 'cosine' 
% "p" = size of the norm when using l_p metrics
period = 1200;
q = 1200;
sigma = 3;
win_len = 3;
metric = 'cosine';
p = 1;
r = 2;

%% Pitch class collection x and n-TETs.
% "x_pc" = pitches (cents) of the tone collection we wish to compare to
% various n-EDOs.
% "x_w" = the respective saliences of the "x_pc" tones. If nothing or 0 is 
% entered for x_w, all saliences are set to 1. 
% "start" is the lowest n n-EDO calculated
% "stop" is the highest n n-EDO calculated
% Note that the size of the period is set in the above Parameters section.
x_pc = [0, 702, 386.3];
x_w = [1, 1, 1];
start = 2;
stop = 100;

%% Calculations for n-TETs and metrics between different expectation arrays
d = zeros(1, stop - start + 1);
for n = start:stop
    edon = 0:(1200/n):1200-1;
    d(n - start + 1)...
        = ExpectationMetrics(period,q,metric,p,sigma,win_len,x_pc,x_w,edon,y_w,'rel',r);
end

%% Plotting
figure(4), subplot(3,1,3), stairs(start:stop,d)
axis([1,stop,0.5,1.1])
set(gca,'XTick',0:5:stop)
